@@ -10,9 +10,11 @@ from zipfile import is_zipfile
 import click
 import requests
 from elftools.elf.elffile import ELFFile
-from requests import Response
+from requests import Response, Session
+from requests.adapters import HTTPAdapter
 from tqdm import tqdm
 from tqdm.utils import CallbackIOWrapper
+from urllib3 import Retry
 
 from .authenticator import Authenticator
 from .context import MemfaultCliClickContext, PackageInfo
@@ -37,6 +39,21 @@ class Uploader:
         self.ctx: MemfaultCliClickContext = ctx
         self.file_path = file_path
         self.authenticator: Authenticator = authenticator
+        self.session = self._create_requests_session()
+
+    @staticmethod
+    def _create_requests_session() -> Session:
+        retry_strategy = Retry(
+            total=5,
+            backoff_factor=2,  # Sleep for 2s, 4s, 8s, 16s, 32s, Stop
+            status_forcelist=[429, 500, 502, 503, 504],
+            method_whitelist=["HEAD", "GET", "POST", "PUT"],
+        )
+        adapter = HTTPAdapter(max_retries=retry_strategy)
+        session = requests.Session()
+        session.mount("https://", adapter)
+        session.mount("http://", adapter)
+        return session
 
     @staticmethod
     @abstractmethod
@@ -81,7 +98,7 @@ class Uploader:
         return False
 
     def _prepare_upload(self) -> Tuple[str, str]:
-        response = requests.post(self.upload_url(), **self.authenticator.requests_auth_params())
+        response = self.session.post(self.upload_url(), **self.authenticator.requests_auth_params())
         check_response(response)
         data = response.json()["data"]
         return data["upload_url"], data["token"]
@@ -98,7 +115,8 @@ class Uploader:
                 disable=not progressbar,
             ) as t:
                 wrapped_file = CallbackIOWrapper(t.update, file, "read")
-                check_response(requests.put(upload_url, data=wrapped_file))
+                response = self.session.put(upload_url, data=wrapped_file)
+                check_response(response)
 
     def post_token_extra_request_data(self) -> Optional[Dict]:
         return None
@@ -109,7 +127,7 @@ class Uploader:
         if extra_request_data:
             json_d.update(extra_request_data)
         check_response(
-            requests.post(
+            self.session.post(
                 self.entity_url(), json=json_d, **self.authenticator.requests_auth_params()
             )
         )
@@ -189,7 +207,7 @@ class SymbolUploader(Uploader):
         return True
 
     def _is_already_uploaded(self) -> bool:
-        response = requests.head(
+        response = self.session.head(
             f"{self._projects_base_url()}/symbols-by-gnu-build-id/{self.gnu_build_id}",
             auth=self.ctx.account,
         )
